/****************************************************************************
 * @file     main.c
 * @version  V1.00
 * @brief    Demonstrate UART transmit and receive function with PDMA
 *
 * @copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M480.h"

#define PLL_CLOCK   192000000

#define ENABLE_PDMA_INTERRUPT 1
#define PDMA_TEST_LENGTH 1
#define DATA_TEST_LENGTH 100

/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
static uint8_t g_u8Tx_Buffer[DATA_TEST_LENGTH];
static uint8_t g_u8Rx_Buffer[DATA_TEST_LENGTH];

volatile uint32_t u32IsTestOver = 0;
volatile uint32_t u32Counter = 0;

uint8_t g_au32SrcArray0[8] = {0x1};
uint32_t g_u32DMAConfig = 0;
uint8_t u32PDMADone = 0;
uint32_t i;

typedef struct dma_desc_t {
    uint32_t ctl;
    uint32_t src;
    uint32_t dest;
    uint32_t offset;
} DMA_DESC_T;

DMA_DESC_T DMA_DESC[3];

/*---------------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void);
void PDMA_IRQHandler(void);
void UART1_IRQHandler(void);
void UART_PDMATest(void);

/**
 * @brief       DMA IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The DMA default IRQ, declared in startup_M480.s.
 */
void PDMA_IRQHandler(void)
{
	u32PDMADone ++;
	
    /* Clear transfer done flag of channel 4 */
    PDMA_CLR_TD_FLAG(PDMA,PDMA_TDSTS_TDIF1_Msk);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC clock */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Wait for HIRC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Enable HXT clock (external XTAL 12MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Wait for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(192000000);
    CLK->PCLKDIV = (CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2); // PCLK divider set 2

    /* Enable IP clock */
    CLK_EnableModuleClock(UART0_MODULE);
	CLK_EnableModuleClock(UART1_MODULE);
    CLK_EnableModuleClock(PDMA_MODULE);
    CLK_EnableModuleClock(EADC_MODULE);
	CLK_EnableModuleClock(EPWM0_MODULE);

    /* Select IP clock source */
    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HIRC, CLK_CLKDIV0_UART0(1));
		CLK_SetModuleClock(UART1_MODULE, CLK_CLKSEL1_UART1SEL_HIRC, CLK_CLKDIV0_UART1(1));

    /* Select EPWM0 module clock source as PCLK0 */
    CLK_SetModuleClock(EPWM0_MODULE, CLK_CLKSEL2_EPWM0SEL_PCLK0, 0);

    /* EADC clock source is 80MHz, set divider to 8, ADC clock is 80/8 MHz */
    CLK_SetModuleClock(EADC_MODULE, 0, CLK_CLKDIV0_EADC(8));

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PD multi-function pins for UART0 RXD and TXD */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD2MFP_Msk | SYS_GPD_MFPL_PD3MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD2MFP_UART0_RXD | SYS_GPD_MFPL_PD3MFP_UART0_TXD);

    /* Set PA multi-function pins for UART1 TXD, RXD */
    SYS->GPA_MFPL &= ~(SYS_GPA_MFPL_PA2MFP_Msk | SYS_GPA_MFPL_PA3MFP_Msk);
    SYS->GPA_MFPL |= (SYS_GPA_MFPL_PA2MFP_UART1_RXD | SYS_GPA_MFPL_PA3MFP_UART1_TXD);


    /* Lock protected registers */
    SYS_LockReg();

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART0 */
    SYS_ResetModule(UART0_RST);
	
    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);

}

void IDLE_UARTFunction(void)
{

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART1                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART module */
    SYS_ResetModule(UART1_RST);
    UART_Open(UART1, 115200);
	UART1->INTEN |= UART_INTEN_TXPDMAEN_Msk;
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init EPWM0                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset EPWM0 module */
    SYS_ResetModule(EPWM0_RST);

    /* Set EPWM0 timer clock prescaler */
    EPWM_SET_PRESCALER(EPWM0, 0, 0);

    /* Set up counter type */
    EPWM0->CTL1 &= ~EPWM_CTL1_CNTTYPE0_Msk;

    // EPWM0 channel 0 frequency is 10Hz, duty 30%,
    EPWM_ConfigOutputChannel(EPWM0, 0, 10, 30);

    /* EPWM period point trigger ADC enable */
    EPWM_EnableADCTrigger(EPWM0, 0, EPWM_TRG_ADC_EVEN_PERIOD);

    /* Set output level at zero, compare up, period(center) and compare down of specified channel */
    EPWM_SET_OUTPUT_LEVEL(EPWM0, BIT0, EPWM_OUTPUT_HIGH, EPWM_OUTPUT_LOW, EPWM_OUTPUT_NOTHING, EPWM_OUTPUT_NOTHING);

    /* Enable output of EPWM0 channel 0 */
    EPWM_EnableOutput(EPWM0, BIT0);

	/*---------------------------------------------------------------------------------------------------------*/
    /* Init EADC                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
	/* Set input mode as single-end and enable the A/D converter */
	EADC_Open(EADC, EADC_CTL_DIFFEN_SINGLE_END);

	/* Configure the sample module 0 for analog input channel 2 and enable EPWM0 trigger source */
	EADC_ConfigSampleModule(EADC, 0, EADC_PWM0TG0_TRIGGER, 2);
	
	EADC->CTL &= 0xFFFFF3F; 
	EADC->CTL |= 1<<6; 
	
	EADC_ENABLE_PDMA(EADC);
	
	/* Enable EPWM0 channel 0 counter */
	EPWM_Start(EPWM0, BIT0); /* EPWM0 channel 0 counter start running. */	

	/*---------------------------------------------------------------------------------------------------------*/
    /* Init PDMA                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Open Channel 1, 4 */
    PDMA_Open(PDMA, 1 << 1);

    /* Enable Scatter Gather mode, assign the first scatter-gather descriptor table is table 1,
       and set transfer mode as memory to peripheral */
	PDMA_SetTransferMode(PDMA, 1, PDMA_ADC_RX, TRUE, (uint32_t)&DMA_DESC[0]);


    /* Scatter-Gather descriptor table configuration in SRAM */
    g_u32DMAConfig = \
                     (0 << PDMA_DSCT_CTL_TXCNT_Pos) | /* Transfer count is 1 */ \
                     PDMA_WIDTH_32 |  /* Transfer width is 32 bits(one word) */ \
                     PDMA_SAR_FIX |   /* Source increment size is fixed(no increment) */ \
                     PDMA_DAR_FIX |   /* Destination increment size is fixed(no increment) */ \
                     PDMA_REQ_SINGLE | /* Transfer type is burst transfer type */ \
                     PDMA_BURST_4 |   /* Burst size is 128. No effect in single transfer type */ \
                     PDMA_OP_SCATTER; /* Operation mode is scatter-gather mode */

    /*------------------------------------------------------------------------------------------------------
						Scatter Gather mode table 1
		------------------------------------------------------------------------------------------------------*/

    DMA_DESC[0].ctl = g_u32DMAConfig;
    /* Configure source address */
    DMA_DESC[0].src = EADC_BASE; /* Ping-Pong buffer 1 */
    /* Configure destination address */
    DMA_DESC[0].dest = ((uint32_t) (&g_u8Tx_Buffer[0]));
    /* Configure next descriptor table address */
    DMA_DESC[0].offset = (uint32_t)&DMA_DESC[1] - (PDMA->SCATBA); /* next operation table is table 2 */


    /*------------------------------------------------------------------------------------------------------
						Scatter Gather mode table 2
		------------------------------------------------------------------------------------------------------*/

    DMA_DESC[1].ctl = g_u32DMAConfig;
    /* Configure source address */
    DMA_DESC[1].src = ((uint32_t) (&g_u8Tx_Buffer[0])); /* Ping-Pong buffer 1 */
    /* Configure destination address */
    DMA_DESC[1].dest = ((uint32_t) (&g_u8Rx_Buffer[0]));
    /* Configure next descriptor table address */
    DMA_DESC[1].offset = (uint32_t)&DMA_DESC[2] - (PDMA->SCATBA); /* next operation table is table 2 */
		
    /*------------------------------------------------------------------------------------------------------
						Scatter Gather mode table 3
		------------------------------------------------------------------------------------------------------*/

    DMA_DESC[2].ctl = g_u32DMAConfig;
    /* Configure source address */
    DMA_DESC[2].src = ((uint32_t) (&g_u8Rx_Buffer[0])); /* Ping-Pong buffer 1 */
    /* Configure destination address */
    DMA_DESC[2].dest = UART1_BASE;
    /* Configure next descriptor table address */
    DMA_DESC[2].offset = 0; /* next operation table is table 2 */

    /* Enable transfer done interrupt */
    PDMA_EnableInt(PDMA, 1, PDMA_INT_TRANS_DONE);
    NVIC_EnableIRQ(PDMA_IRQn);

    printf("\n\nCPU @ %dHz\n", SystemCoreClock);
    printf("+-----------------------------------------------------------------------+ \n");
    printf("|    M480 PDMA Driver Ping-Pong Buffer Sample Code (Scatter-gather)     | \n");
    printf("+-----------------------------------------------------------------------+ \n");


}

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int main(void)
{
	
    /* Init System, IP clock and multi-function I/O */
    SYS_Init();
    /* Init UART0 for printf */
    UART0_Init();
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* SAMPLE CODE                                                                                             */
    /*---------------------------------------------------------------------------------------------------------*/

    printf("\n\nCPU @ %dHz\n", SystemCoreClock);

    IDLE_UARTFunction();

    while(1)
	{
		
		if(u32PDMADone == 4)
		{

				PDMA_Open(PDMA, 1 << 1);			
			
				/* Enable Scatter Gather mode, assign the first scatter-gather descriptor table is table 1,
					and set transfer mode as memory to peripheral */
				PDMA_SetTransferMode(PDMA, 1, PDMA_ADC_RX, TRUE, (uint32_t)&DMA_DESC[0]);
				/*---------------------------------------------------------------------------------------------------------*/
				/* Init EADC                                                                                               */
				/*---------------------------------------------------------------------------------------------------------*/
				EADC_Close(EADC);
				/* Set input mode as single-end and enable the A/D converter */
				EADC_Open(EADC, EADC_CTL_DIFFEN_SINGLE_END);

				/* Configure the sample module 0 for analog input channel 2 and enable EPWM0 trigger source */
				EADC_ConfigSampleModule(EADC, 0, EADC_PWM0TG0_TRIGGER, 2);
				
				EADC->CTL &= 0xFFFFF3F; 
				EADC->CTL |= 1<<6; 
				
				EADC_ENABLE_PDMA(EADC);
						
				u32PDMADone = 1;

	
		}
	}
	
}


